% File src/library/base/man/ifelse.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2015 R Core Team
% Distributed under GPL 2 or later

\name{ifelse}
\title{Conditional Element Selection}
\usage{
ifelse(test, yes, no)
}
\alias{ifelse}
\description{
  \code{ifelse} は \code{test} と同じ形の値を返す．
  それは \code{test} の要素が \code{TRUE} か \code{FALSE} かに応じて
  要素 \code{yes} か \code{no} で埋められている．
}
\arguments{
  \item{test}{論理値モードに強制変換できるオブジェクト．}
  \item{yes}{\code{test} の真の要素に対する返り値．}
  \item{no}{\code{test} の偽の要素に対する返り値．}
}
\details{
  もし \code{yes} や \code{no} が短すぎるとそれらの要素はリサイクルされる．
  \code{yes} は \code{test} の要素が真である時，そしてその時だけ評価される．
  \code{no} についても同様である．

  \code{test} 中の欠損値は結果中の欠損値を与える．
}
\value{
  \code{x} と同じ長さと属性のベクトル(次元と \code{"class"} を含む)で，
  データの値は \code{yes} と \code{no} から取られる．解答のモードは
  論理値から先ず \code{x} から取られた任意の値，そしてそれから
  \code{y} から取られた任意の値に合わせて強制変換される．
}
\section{警告}{
  結果のモードは \code{test} の値に依存するかもしれない(例を見よ)．
  そして結果のクラス属性(\code{\link{oldClass}} を見よ)は
  \code{test} から取られ，\code{yes} や \code{no} から選択された値に対して
  不適切かもしれない．

  可能性として \code{test} 中の欠損値を処理するように拡張された
  次のような構成を使うほうが好ましいことがある
\preformatted{  (tmp <- yes; tmp[!test] <- no[!test]; tmp)
}

  更に \code{test} の結果が単純な真/偽，つまり
  \code{length(test) == 1}，の時は
  コード\cr
  \code{if(test) yes else no} は相当効率的であり
  しばしば \code{ifelse(test, yes, no)} より好ましい．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\seealso{
\code{\link{if}}.
}
\examples{
x <- c(6:-4)
sqrt(x)  #- 警告が出る
sqrt(ifelse(x >= 0, x, NA))  # 警告なし

## 注意：次もまた警告がでる!
ifelse(x >= 0, sqrt(x), NA)


## ifelse() は属性を取り去る
## これは日付や因子を扱う時重要になる
x <- seq(as.Date("2000-02-29"), as.Date("2004-10-04"), by = "1 month")
## 多くの "yyyy-mm-29"，しかし閏年に僅かな "yyyy-03-01"
y <- ifelse(as.POSIXlt(x)$mday == 29, x, NA)
head(y) # 期待されるものではない ... ==> クラス属性を再現する必要：
class(y) <- class(x)
y
## ==> 再び ifelse() を使わないほうが良いケース，しかし
## どちらもより効率的でわかりやすい：
y2 <- x
y2[as.POSIXlt(x)$mday != 29] <- NA
stopifnot(identical(y2, y))


## 異なった返り値モードの例：
yes <- 1:3
no <- pi^(0:3)
typeof(ifelse(NA,    yes, no)) # 論理値
typeof(ifelse(TRUE,  yes, no)) # 整数
typeof(ifelse(FALSE, yes, no)) # 倍精度実数
}
\keyword{logic}
\keyword{programming}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/ifelse.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2015 R Core Team
%% Distributed under GPL 2 or later
%
%\name{ifelse}
%\title{Conditional Element Selection}
%\usage{
%ifelse(test, yes, no)
%}
%\alias{ifelse}
%\description{
%  \code{ifelse} returns a value with the same shape as
%  \code{test} which is filled with elements selected
%  from either \code{yes} or \code{no}
%  depending on whether the element of \code{test}
%  is \code{TRUE} or \code{FALSE}.
%}
%\arguments{
%  \item{test}{an object which can be coerced to logical mode.}
%  \item{yes}{return values for true elements of \code{test}.}
%  \item{no}{return values for false elements of \code{test}.}
%}
%\details{
%  If \code{yes} or \code{no} are too short, their elements are recycled.
%  \code{yes} will be evaluated if and only if any element of \code{test}
%  is true, and analogously for \code{no}.
%
%  Missing values in \code{test} give missing values in the result.
%}
%\value{
%  A vector of the same length and attributes (including dimensions and
%  \code{"class"}) as \code{test} and data values from the values of
%  \code{yes} or \code{no}.  The mode of the answer will be coerced from
%  logical to accommodate first any values taken from \code{yes} and then
%  any values taken from \code{no}.
%}
%\section{Warning}{
%  The mode of the result may depend on the value of \code{test} (see the
%  examples), and the class attribute (see \code{\link{oldClass}}) of the
%  result is taken from \code{test} and may be inappropriate for the
%  values selected from \code{yes} and \code{no}.
%
%  Sometimes it is better to use a construction such as
%\preformatted{  (tmp <- yes; tmp[!test] <- no[!test]; tmp)
%}, possibly extended to handle missing values in \code{test}.
%
%  Further note that \code{if(test) yes else no}  is much more efficient
%  and often much preferable to \code{ifelse(test, yes, no)} whenever
%  \code{test} is a simple true/false result, i.e., when
%  \code{length(test) == 1}.
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%\seealso{
%\code{\link{if}}.
%}
%\examples{
%x <- c(6:-4)
%sqrt(x)  #- gives warning
%sqrt(ifelse(x >= 0, x, NA))  # no warning
%
%## Note: the following also gives the warning !
%ifelse(x >= 0, sqrt(x), NA)
%
%
%## ifelse() strips attributes
%## This is important when working with Dates and factors
%x <- seq(as.Date("2000-02-29"), as.Date("2004-10-04"), by = "1 month")
%## has many "yyyy-mm-29", but a few "yyyy-03-01" in the non-leap years
%y <- ifelse(as.POSIXlt(x)$mday == 29, x, NA)
%head(y) # not what you expected ... ==> need restore the class attribute:
%class(y) <- class(x)
%y
%## ==> Again a case where it is better *not* to use ifelse(), but
%## both more efficient and clear:
%y2 <- x
%y2[as.POSIXlt(x)$mday != 29] <- NA
%stopifnot(identical(y2, y))
%
%
%## example of different return modes:
%yes <- 1:3
%no <- pi^(0:3)
%typeof(ifelse(NA,    yes, no)) # logical
%typeof(ifelse(TRUE,  yes, no)) # integer
%typeof(ifelse(FALSE, yes, no)) # double
%}
%\keyword{logic}
%\keyword{programming}
%
\encoding{UTF8}
