% File src/library/base/man/strsplit.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2015 R Core Team
% Distributed under GPL 2 or later

\name{strsplit}
\alias{strsplit}
\title{Split the Elements of a Character Vector}
\description{
  文字列ベクトル \code{x} の要素を，それらの内部での
  部分文字列 \code{split} へのマッチに従って，
  部分文字列に分割する．
%  Split the elements of a character vector \code{x} into substrings
%  according to the matches to substring \code{split} within them.
}
\usage{
strsplit(x, split, fixed = FALSE, perl = FALSE, useBytes = FALSE)
}
\arguments{
  \item{x}{
    文字列ベクトルで，その各要素を分割する．
    因子を含む他の入力はエラーを与える．
  }
  \item{split}{
    分割に対して使われる正規表現を含む(\code{fixed = TRUE} で無い限り)
    文字列(またはそうしたものへ変換できるオブジェクト)．
    もしマッチが無ければ，特に 
    \code{split} が長さゼロなら，\code{x} は単一の文字に分解される．
    もし \code{split} が 1 より大きな長さを持てば，それは \code{x} に
    沿ってリサイクルされる．
  }
  \item{fixed}{
    論理値．もし \code{TRUE} なら \code{split} に正確にマッチする．
    さもなければ正規表現を使う．\code{perl} に優先する．
  }
  \item{perl}{論理値．Perl 互換の正規表現を使うか?}
  \item{useBytes}{論理値．もし \code{TRUE} ならマッチングは文字毎ではなく
    バイト毎に行われ，マークされたエンコーディングを持つ入力は
    変換されない．これは \code{"bytes"} とマークされた入力が一つでも
    見つかれば強制される(\code{\link{Encoding}} を見よ)．}
}
\details{
  引数 \code{split} は文字列に変換されるので，下の例のように
  \code{split = NULL} の使用は\cr
  \code{split = character(0)} を意味するのが分かる，

  単一の文字への分割は
  \code{split = character(0)} 又は \code{split = ""} で
  行える；二つは同値である．
  ここでの\sQuote{文字}の定義はロケールに依存する：
  単バイトロケールではそれはバイトであり，多バイトロケールでは
  それは\sQuote{幅広文字}で表現される単位である
  (ほとんど常に Unicode の code point)．

  \code{split} の欠損値は \code{x} の対応要素を全く分割しない．

  各入力文字列に適用されるアルゴリズムは
\preformatted{    repeat \{
        もし文字列が空なら
            中断．
        もしマッチが一つあれば
            マッチの左側の文字列を出力に加える．
            マッチとその左側の全てを削除する．
        さもなければ
            文字列を出力に加える．
            中断．
    \}
}
  これはもしマッチが(空でない)文字列の最初であれば
  出力の最初の要素は \code{""} であるが，もし文字列の最後でマッチがあれば
  出力はマッチ部分を除いたものと一致することを注意する．

  現在のロケールで不正な入力は最大5回まで警告される．
}
\value{
  \code{x} と同じ長さのリストで，その \code{i} 番目の要素は
  \code{x[i]} を分割したベクトルを含む．

  もし \code{x} か \code{split} のどれかの要素が UTF-8 と
  宣言されていれば(\code{\link{Encoding}} を見よ)，結果中の全ての非アスキー
  文字列は UTF-8 とされ，それらのエンコーディングは UTF-8 と宣言される．
  \code{perl = TRUE, useBytes = FALSE} に対しては多バイトロケールの
  非アスキー文字列は UTF-8 に翻訳される．
}

\seealso{
  逆操作は \code{\link{paste}}．
  文字列の探索と処理には \code{\link{grep}} と \code{\link{sub}}；
  また \code{\link{nchar}}, \code{\link{substr}}．

  パターンの指定の詳細については\sQuote{正規表現}．
}
\examples{
noquote(strsplit("A text I want to display with spaces", NULL)[[1]])

x <- c(as = "asfef", qu = "qwerty", "yuiop[", "b", "stuff.blah.yech")
# x を文字 e について分割
strsplit(x, "e")

unlist(strsplit("a.b.c", "."))
## [1] "" "" "" "" ""
## 'split' は正規表現であることを注意!
## もし本当に '.' に関して分割したければ次を使う
unlist(strsplit("a.b.c", "[.]"))
## [1] "a" "b" "c"
## 又は
unlist(strsplit("a.b.c", ".", fixed = TRUE))

## 有用な関数：文字列に対する rev()
strReverse <- function(x)
        sapply(lapply(strsplit(x, NULL), rev), paste, collapse = "")
strReverse(c("abc", "Statistics"))

## R の中核メンバーのファーストネームを得る
a <- readLines(file.path(R.home("doc"),"AUTHORS"))[-(1:8)]
a <- a[(0:2)-length(a)]
(a <- sub(" .*","", a))
# そしてそれらを逆転する
strReverse(a)

## 最後の空文字列が生成されないことを注意：
strsplit(paste(c("", "a", ""), collapse="#"), split="#")[[1]]
# [1] ""  "a"
## そしてまた空文字列は明確なマッチの前でだけ生成される
strsplit("", " ")[[1]]    # character(0)
strsplit(" ", " ")[[1]]   # [1] ""
}
\keyword{character}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/strsplit.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2015 R Core Team
%% Distributed under GPL 2 or later
%
%\name{strsplit}
%\alias{strsplit}
%\title{Split the Elements of a Character Vector}
%\description{
%  Split the elements of a character vector \code{x} into substrings
%  according to the matches to substring \code{split} within them.
%}
%\usage{
%strsplit(x, split, fixed = FALSE, perl = FALSE, useBytes = FALSE)
%}
%\arguments{
%  \item{x}{
%    character vector, each element of which is to be split.  Other
%    inputs, including a factor, will give an error.
%  }
%  \item{split}{
%    character vector (or object which can be coerced to such)
%    containing \link{regular expression}(s) (unless \code{fixed = TRUE})
%    to use for splitting.  If empty matches occur, in particular if
%    \code{split} has length 0, \code{x} is split into single characters.
%    If \code{split} has length greater than 1, it is re-cycled along
%    \code{x}.
%  }
%  \item{fixed}{
%    logical.  If \code{TRUE} match \code{split} exactly, otherwise
%    use regular expressions.  Has priority over \code{perl}.
%  }
%  \item{perl}{logical.  Should Perl-compatible regexps be used?}
%  \item{useBytes}{logical.  If \code{TRUE} the matching is done
%    byte-by-byte rather than character-by-character, and inputs with
%    marked encodings are not converted.  This is forced (with a warning)
%    if any input is found which is marked as \code{"bytes"}
%    (see \code{\link{Encoding}}).}
%}
%\details{
%  Argument \code{split} will be coerced to character, so
%  you will see uses with \code{split = NULL} to mean
%  \code{split = character(0)}, including in the examples below.
%
%  Note that splitting into single characters can be done \emph{via}
%  \code{split = character(0)} or \code{split = ""}; the two are
%  equivalent.  The definition of \sQuote{character} here depends on the
%  locale: in a single-byte locale it is a byte, and in a multi-byte
%  locale it is the unit represented by a \sQuote{wide character} (almost
%  always a Unicode code point).
%
%  A missing value of \code{split} does not split the corresponding
%  element(s) of \code{x} at all.
%
%  The algorithm applied to each input string is
%\preformatted{    repeat \{
%        if the string is empty
%            break.
%        if there is a match
%            add the string to the left of the match to the output.
%            remove the match and all to the left of it.
%        else
%            add the string to the output.
%            break.
%    \}
%}
%  Note that this means that if there is a match at the beginning of a
%  (non-empty) string, the first element of the output is \code{""}, but
%  if there is a match at the end of the string, the output is the same
%  as with the match removed.
%
%  Invalid inputs in the current locale are warned about up to 5 times.
%}
%\value{
%  A list of the same length as \code{x}, the \code{i}-th element of which
%  contains the vector of splits of \code{x[i]}.
%
%  If any element of \code{x} or \code{split} is declared to be in UTF-8
%  (see \code{\link{Encoding}}), all non-ASCII character strings in the
%  result will be in UTF-8 and have their encoding declared as UTF-8.
%  For \code{perl = TRUE, useBytes = FALSE} all non-ASCII strings in a
%  multibyte locale are translated to UTF-8.
%}
%
%\seealso{
%  \code{\link{paste}} for the reverse,
%  \code{\link{grep}} and \code{\link{sub}} for string search and
%  manipulation; also \code{\link{nchar}}, \code{\link{substr}}.
%
%  \sQuote{\link{regular expression}} for the details of the pattern
%  specification.
%}
%\examples{
%noquote(strsplit("A text I want to display with spaces", NULL)[[1]])
%
%x <- c(as = "asfef", qu = "qwerty", "yuiop[", "b", "stuff.blah.yech")
%# split x on the letter e
%strsplit(x, "e")
%
%unlist(strsplit("a.b.c", "."))
%## [1] "" "" "" "" ""
%## Note that 'split' is a regexp!
%## If you really want to split on '.', use
%unlist(strsplit("a.b.c", "[.]"))
%## [1] "a" "b" "c"
%## or
%unlist(strsplit("a.b.c", ".", fixed = TRUE))
%
%## a useful function: rev() for strings
%strReverse <- function(x)
%        sapply(lapply(strsplit(x, NULL), rev), paste, collapse = "")
%strReverse(c("abc", "Statistics"))
%
%## get the first names of the members of R-core
%a <- readLines(file.path(R.home("doc"),"AUTHORS"))[-(1:8)]
%a <- a[(0:2)-length(a)]
%(a <- sub(" .*","", a))
%# and reverse them
%strReverse(a)
%
%## Note that final empty strings are not produced:
%strsplit(paste(c("", "a", ""), collapse="#"), split="#")[[1]]
%# [1] ""  "a"
%## and also an empty string is only produced before a definite match:
%strsplit("", " ")[[1]]    # character(0)
%strsplit(" ", " ")[[1]]   # [1] ""
%}
%\keyword{character}
%
\encoding{UTF8}
