% File src/library/base/man/substitute.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2011 R Core Team
% Distributed under GPL 2 or later

\name{substitute}
\title{Substituting and Quoting Expressions}
\usage{
substitute(expr, env)
quote(expr)
enquote(cl)
}
\alias{substitute}
\alias{quote}
\alias{enquote}
\description{
  \code{substitute} は(未評価)表現式 \code{expr} に対する構文解析木を返す．
  \code{env} 中に拘束された任意の変数が代入されている．

  \code{quote} は単にその引数を返す．引数は評価されず任意の R 表現式で良い．

  \code{enquote} は
  \code{Foo(....)} の形式の呼び出しを呼び出し \code{quote(Foo(....))} に
  変換する単純な一行ユーティリティである．
  これは典型的に \code{\link{call}} を早期の評価から守るために使われる．
}
\arguments{
  \item{expr}{構文的に適正な \R 表現式．}
  \item{cl}{\code{\link{call}}，つまり
    クラス(そしてモード)が \code{"call"} である \R オブジェクト．}
  \item{env}{環境かリストオブジェクト．既定は現在の評価環境．}
}
\details{
  \code{substitute} の典型的な用法はデータセットとプロットのために
  情報開示的なラベルを作ることである．
  下の \code{myplot} 例はこの機能の簡単な用法を示す．
  これは関数 \code{\link{deparse}} と \code{substitute} を使い
  関数 \code{myplot} への実際の引数の文字列バージョンであるプロット用
  ラベルを作る．

  代入は構文解析木の各要素を調べることで次のように行われる：
  もし \code{env} に拘束されたシンボルでなければそのままにする．
  もしそれが予約オブジェクト，つまり関数への形式的引数か
  \code{\link{delayedAssign}()} を使って明示的に作られた，
  ならば，予約の表現式スロットでシンボルを置き換える．
  もしそれが通常の変数ならその値が代入される．例外は
  \code{env} が \code{\link{.GlobalEnv}} の時で，その場合は
  シンボルは変更されない．

  \code{quote} と \code{substitute} は共に\sQuote{特殊}なプリミティブ関数で
  それらの引数を評価しない．
}
\value{
  結果の \code{\link{mode}} は一般に \code{"call"} であるが，
  原理的には任意のタイプであり得る．特に変数が一つの表現式は
  モード \code{"name"} を持ち，定数は適当な基礎モードを持つ．
}
\note{
  \code{substitute} は純粋に辞書式の基礎の上で動作する．
  結果の表現式が意味を持つ保証は無い．

  引数が \code{expression(\dots)} の時
  代入と引用化はしばしば混乱を招く．
  結果は \code{\link{expression}} 構成関数への呼び出しであり，
  実際の表現式オブジェクトを与えるには \code{\link{eval}} で
  評価される必要がある．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\seealso{
  引数 \sQuote{missingness} については \code{\link{missing}}，
  部分代入については \code{\link{bquote}}，
  文字列に引用符を加えるには \code{\link{sQuote}} と \code{\link{dQuote}}．

  表現式や呼び出しからシンボル名を取り出すには \code{\link{all.names}}．
}
\examples{
require(graphics)
(s.e <- substitute(expression(a + b), list(a = 1)))  #> expression(1 + b)
(s.s <- substitute( a + b,            list(a = 1)))  #> 1 + b
c(mode(s.e), typeof(s.e)) #  "call", "language"
c(mode(s.s), typeof(s.s)) #   (同じ)
# しかし：
(e.s.e <- eval(s.e))          #>  expression(1 + b)
c(mode(e.s.e), typeof(e.s.e)) #  "expression", "expression"

substitute(x <- x + 1, list(x = 1)) # 無意味

myplot <- function(x, y)
    plot(x, y, xlab = deparse(substitute(x)),
         ylab = deparse(substitute(y)))

## 遅延評価の簡単な例，等：

f1 <- function(x, y = x)             { x <- x + 1; y }
s1 <- function(x, y = substitute(x)) { x <- x + 1; y }
s2 <- function(x, y) { if(missing(y)) y <- substitute(x); x <- x + 1; y }
a <- 10
f1(a)  # 11
s1(a)  # 11
s2(a)  # a
typeof(s2(a))  # "symbol"
}
\keyword{programming}
\keyword{data}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/substitute.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2011 R Core Team
%% Distributed under GPL 2 or later
%
%\name{substitute}
%\title{Substituting and Quoting Expressions}
%\usage{
%substitute(expr, env)
%quote(expr)
%enquote(cl)
%}
%\alias{substitute}
%\alias{quote}
%\alias{enquote}
%\description{
%  \code{substitute} returns the parse tree for the (unevaluated)
%  expression \code{expr}, substituting any variables bound in
%  \code{env}.
%
%  \code{quote} simply returns its argument. The argument is not evaluated
%  and can be any R expression.
%
%  \code{enquote} is a simple one-line utility which transforms a call of
%  the form \code{Foo(....)} into the call \code{quote(Foo(....))}.  This
%  is typically used to protect a \code{\link{call}} from early evaluation.
%}
%\arguments{
%  \item{expr}{any syntactically valid \R expression}
%  \item{cl}{a \code{\link{call}}, i.e., an \R object of
%    \code{\link{class}} (and \code{\link{mode}}) \code{"call"}.}
%  \item{env}{an environment or a list object.  Defaults to the
%    current evaluation environment.}
%}
%\details{
%  The typical use of \code{substitute} is to create informative labels
%  for data sets and plots.
%  The \code{myplot} example below shows a simple use of this facility.
%  It uses the functions \code{\link{deparse}} and \code{substitute}
%  to create labels for a plot which are character string versions
%  of the actual arguments to the function \code{myplot}.
%
%  Substitution takes place by examining each component of the parse tree
%  as follows: If it is not a bound symbol in \code{env}, it is
%  unchanged.  If it is a promise object, i.e., a formal argument to a
%  function or explicitly created using \code{\link{delayedAssign}()},
%  the expression slot of the promise replaces the symbol.  If it is an
%  ordinary variable, its value is substituted, unless \code{env} is
%  \code{\link{.GlobalEnv}} in which case the symbol is left unchanged.
%
%  Both \code{quote} and \code{substitute} are \sQuote{special}
%  \link{primitive} functions which do not evaluate their arguments.
%}
%\value{
%  The \code{\link{mode}} of the result is generally \code{"call"} but
%  may in principle be any type. In particular, single-variable
%  expressions have mode \code{"name"} and constants have the
%  appropriate base mode.
%}
%\note{
%  \code{substitute} works on a purely lexical basis.  There is no
%  guarantee that the resulting expression makes any sense.
%
%  Substituting and quoting often cause confusion when the argument is
%  \code{expression(\dots)}.  The result is a call to the
%  \code{\link{expression}} constructor function and needs to be evaluated
%  with \code{\link{eval}} to give the actual expression object.
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%\seealso{
%  \code{\link{missing}} for argument \sQuote{missingness},
%  \code{\link{bquote}} for partial substitution,
%  \code{\link{sQuote}} and \code{\link{dQuote}} for adding quotation
%  marks to strings,
%
%  \code{\link{all.names}} to retrieve the symbol names from an expression
%  or call.
%}
%\examples{
%require(graphics)
%(s.e <- substitute(expression(a + b), list(a = 1)))  #> expression(1 + b)
%(s.s <- substitute( a + b,            list(a = 1)))  #> 1 + b
%c(mode(s.e), typeof(s.e)) #  "call", "language"
%c(mode(s.s), typeof(s.s)) #   (the same)
%# but:
%(e.s.e <- eval(s.e))          #>  expression(1 + b)
%c(mode(e.s.e), typeof(e.s.e)) #  "expression", "expression"
%
%substitute(x <- x + 1, list(x = 1)) # nonsense
%
%myplot <- function(x, y)
%    plot(x, y, xlab = deparse(substitute(x)),
%         ylab = deparse(substitute(y)))
%
%## Simple examples about lazy evaluation, etc:
%
%f1 <- function(x, y = x)             { x <- x + 1; y }
%s1 <- function(x, y = substitute(x)) { x <- x + 1; y }
%s2 <- function(x, y) { if(missing(y)) y <- substitute(x); x <- x + 1; y }
%a <- 10
%f1(a)  # 11
%s1(a)  # 11
%s2(a)  # a
%typeof(s2(a))  # "symbol"
%}
%\keyword{programming}
%\keyword{data}
%
\encoding{UTF8}
