% File src/library/grDevices/man/rgb2hsv.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{rgb2hsv}
\title{RGB 色を HSV 色に変換}
\alias{rgb2hsv}
%%\title{RGB to HSV Conversion}
\concept{color space conversion}
\description{
  \code{rgb2hsv} は色を
  RGB 空間(赤/緑/青)から
  HSV 空間(色調/彩度/値)に変換する．
}
\usage{
rgb2hsv(r, g = NULL, b = NULL, maxColorValue = 255)
}
\arguments{
  \item{r}{\eqn{[0, M]} 中の\sQuote{赤色}の値のベクトル，
    (\eqn{M = }\code{maxColorValue}) 又は3行の rgb 行列．}
  \item{g}{\sQuote{緑色}の値，又は \code{r} が行列ならば
    \code{\link{NULL}}．}
  \item{b}{\sQuote{青色}の値，又は \code{r} が行列ならば
    \code{\link{NULL}}．}
  \item{maxColorValue}{RGB 色値の範囲の最大値を与える数．
    既定の \code{255} は
    \code{\link{col2rgb}()} 中に於けるような典型的な
    範囲 \code{0:255} の RGB コーディングに対応する．}
}
\details{
  値(輝度)は色調の光量を与える．\cr
  色調は優越する波長を記述する．\cr
  彩度は色に混ぜられた色調の量である．

  HSV 色空間は \R では sRGB である RGB 色空間に相対的であり，
  暗黙のうちにガンマ補正されている．
}
\value{
 各色に対する列を持つ行列．行列の三つの行は
 色調，彩度，値 を指示し，
 従って名前 \code{"h"}, \code{"s"} そして \code{"v"} を持つ．
}
\seealso{
  \code{\link{hsv}}, \code{\link{col2rgb}}, \code{\link{rgb}}.
}
\author{
  \R インタフェイスは Wolfram Fischer \email{wolfram@fischer-zim.ch} による;\cr
  C コードは主に Nicholas Lewin-Koh \email{nikko@hailmail.net}．
}
\examples{
## これらは(彩度，明度)は色調だけが異なる
(rc <- col2rgb(c("red", "yellow","green","cyan", "blue", "magenta")))
(hc <- rgb2hsv(rc))
6 * hc["h",] # 色調は等間隔

\testonly{set.seed(151)}
(rgb3 <- floor(256 * matrix(stats::runif(3*12), 3, 12)))
(hsv3 <- rgb2hsv(rgb3))
## 一貫性：
stopifnot(rgb3 == col2rgb(hsv(h = hsv3[1,], s = hsv3[2,], v = hsv3[3,])),
          all.equal(hsv3, rgb2hsv(rgb3/255, maxColorValue = 1)))

## (簡略化された)純 R バージョン -- オリジナルは Wolfram Fischer --
## 正確なアルゴリズムを示す：
rgb2hsvR <- function(rgb, gamma = 1, maxColorValue = 255)
{
    if(!is.numeric(rgb)) stop("rgb matrix must be numeric")
    d <- dim(rgb)
    if(d[1] != 3) stop("rgb matrix must have 3 rows")
    n <- d[2]
    if(n == 0) return(cbind(c(h = 1, s = 1, v = 1))[,0])
    rgb <- rgb/maxColorValue
    if(gamma != 1) rgb <- rgb ^ (1/gamma)

    ## 最大値と最小値を得る
    v <- apply( rgb, 2, max)
    s <- apply( rgb, 2, min)
    D <- v - s # range

    ## 未定義の値に対しては色調をゼロに設定(灰色は色調を持たない)
    h <- numeric(n)
    notgray <- ( s != v )

    ## 青い色調
    idx <- (v == rgb[3,] & notgray )
    if (any (idx))
        h[idx] <- 2/3 + 1/6 * (rgb[1,idx] - rgb[2,idx]) / D[idx]
    ## 緑の色調
    idx <- (v == rgb[2,] & notgray )
    if (any (idx))
        h[idx] <- 1/3 + 1/6 * (rgb[3,idx] - rgb[1,idx]) / D[idx]
    ## 赤の色調
    idx <- (v == rgb[1,] & notgray )
    if (any (idx))
        h[idx] <-       1/6 * (rgb[2,idx] - rgb[3,idx]) / D[idx]

    ## 負の赤に対する修正
    idx <- (h < 0)
    h[idx] <- 1+h[idx]

    ## 彩度設定
    s[! notgray] <- 0;
    s[notgray] <- 1 - s[notgray] / v[notgray]

    rbind( h = h, s = s, v = v )
}

## 同値性を確認：
all.equal(rgb2hsv (rgb3),
          rgb2hsvR(rgb3), tolerance = 1e-14) # TRUE
}
\keyword{color}
\keyword{dplot}

\encoding{UTF-8}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/grDevices/man/rgb2hsv.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{rgb2hsv}
%\alias{rgb2hsv}
%\title{RGB to HSV Conversion}
%\concept{color space conversion}
%\description{
%  \code{rgb2hsv} transforms colors
%  from RGB space (red/green/blue)
%  into HSV space (hue/saturation/value).
%}
%\usage{
%rgb2hsv(r, g = NULL, b = NULL, maxColorValue = 255)
%}
%\arguments{
%  \item{r}{vector of \sQuote{red} values in \eqn{[0, M]}, (\eqn{M
%      = }\code{maxColorValue}) or 3-row rgb matrix.}
%  \item{g}{vector of \sQuote{green} values, or \code{\link{NULL}} when
%    \code{r} is a matrix.}
%  \item{b}{vector of \sQuote{blue} values, or \code{\link{NULL}} when
%    \code{r} is a matrix.}
%  \item{maxColorValue}{number giving the maximum of the RGB color values
%    range.  The default \code{255} corresponds to the typical \code{0:255}
%    RGB coding as in \code{\link{col2rgb}()}.}
%}
%\details{
%  Value (brightness) gives the amount of light in the color.\cr
%  Hue describes the dominant wavelength.\cr
%  Saturation is the amount of Hue mixed into the color.
%
%  An HSV colorspace is relative to an RGB colorspace, which in \R
%  is sRGB, which has an implicit gamma correction.
%}
%\value{
%  A matrix with a column for each color.  The three rows of the matrix
%  indicate hue, saturation and value and are named \code{"h"},
%  \code{"s"}, and \code{"v"} accordingly.
%}
%\seealso{
%  \code{\link{hsv}}, \code{\link{col2rgb}}, \code{\link{rgb}}.
%}
%\author{
%  \R interface by Wolfram Fischer \email{wolfram@fischer-zim.ch};\cr
%  C code mainly by Nicholas Lewin-Koh \email{nikko@hailmail.net}.
%}
%\examples{
%## These (saturated, bright ones) only differ by hue
%(rc <- col2rgb(c("red", "yellow","green","cyan", "blue", "magenta")))
%(hc <- rgb2hsv(rc))
%6 * hc["h",] # the hues are equispaced
%
%\testonly{set.seed(151)}
%(rgb3 <- floor(256 * matrix(stats::runif(3*12), 3, 12)))
%(hsv3 <- rgb2hsv(rgb3))
%## Consistency :
%stopifnot(rgb3 == col2rgb(hsv(h = hsv3[1,], s = hsv3[2,], v = hsv3[3,])),
%          all.equal(hsv3, rgb2hsv(rgb3/255, maxColorValue = 1)))
%
%## A (simplified) pure R version -- originally by Wolfram Fischer --
%## showing the exact algorithm:
%rgb2hsvR <- function(rgb, gamma = 1, maxColorValue = 255)
%{
%    if(!is.numeric(rgb)) stop("rgb matrix must be numeric")
%    d <- dim(rgb)
%    if(d[1] != 3) stop("rgb matrix must have 3 rows")
%    n <- d[2]
%    if(n == 0) return(cbind(c(h = 1, s = 1, v = 1))[,0])
%    rgb <- rgb/maxColorValue
%    if(gamma != 1) rgb <- rgb ^ (1/gamma)
%
%    ## get the max and min
%    v <- apply( rgb, 2, max)
%    s <- apply( rgb, 2, min)
%    D <- v - s # range
%
%    ## set hue to zero for undefined values (gray has no hue)
%    h <- numeric(n)
%    notgray <- ( s != v )
%
%    ## blue hue
%    idx <- (v == rgb[3,] & notgray )
%    if (any (idx))
%        h[idx] <- 2/3 + 1/6 * (rgb[1,idx] - rgb[2,idx]) / D[idx]
%    ## green hue
%    idx <- (v == rgb[2,] & notgray )
%    if (any (idx))
%        h[idx] <- 1/3 + 1/6 * (rgb[3,idx] - rgb[1,idx]) / D[idx]
%    ## red hue
%    idx <- (v == rgb[1,] & notgray )
%    if (any (idx))
%        h[idx] <-       1/6 * (rgb[2,idx] - rgb[3,idx]) / D[idx]
%
%    ## correct for negative red
%    idx <- (h < 0)
%    h[idx] <- 1+h[idx]
%
%    ## set the saturation
%    s[! notgray] <- 0;
%    s[notgray] <- 1 - s[notgray] / v[notgray]
%
%    rbind( h = h, s = s, v = v )
%}
%
%## confirm the equivalence:
%all.equal(rgb2hsv (rgb3),
%          rgb2hsvR(rgb3), tolerance = 1e-14) # TRUE
%}
%\keyword{color}
%\keyword{dplot}
%
%\encoding{UTF-8}
%