% File src/library/stats/man/Wilcoxon.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{Wilcoxon}
\alias{Wilcoxon}
\alias{dwilcox}
\alias{pwilcox}
\alias{qwilcox}
\alias{rwilcox}
\title{Distribution of the Wilcoxon Rank Sum Statistic}
\description{
  それぞれサイズ \code{m} と \code{n} の標本から得られた
  ウィルコクソンのランク和統計量の
  分布に対する密度，分布関数，クォンタイル関数と乱数生成．
}
\usage{
dwilcox(x, m, n, log = FALSE)
pwilcox(q, m, n, lower.tail = TRUE, log.p = FALSE)
qwilcox(p, m, n, lower.tail = TRUE, log.p = FALSE)
rwilcox(nn, m, n)
}
\arguments{
  \item{x, q}{クォンタイルのベクトル．}
  \item{p}{確率のベクトル．}
  \item{n}{観測値数．もし \code{length(n) > 1} なら長さが
    必要な個数とされる．}
  \item{m, n}{それぞれ第一と第二の観測値の数．
    正整数のベクトルで良い．}
  \item{log, log.p}{論理値；もし TRUE なら確率 p は log(p) とされる．}
  \item{lower.tail}{論理値；もし TRUE (既定値)なら
    確率は \eqn{P[X \le x]} で，さもなければ \eqn{P[X > x]}．}
}
\value{
  \code{dwilcox} は密度，\code{pwilcox} は分布関数，
  \code{qwilcox} はクォンタイル関数，そして
  \code{rwilcox} は乱数を生成する．

  不正な引数は値 \code{NaN} を返し，警告が出る．

  \code{rwilcox} に対しては結果の長さは \code{n} で決まり，
  そして他の関数に対しては数値引数の最大長である．

  \code{n} 以外の数値引数は結果の長さまでリサイクルされる．
  論理値引数は最初の要素だけが使われる．
}
\details{
  この分布は次のように得られる．
  \code{x} と \code{y} を
  サイズが \code{m} と \code{n} の二つのランダム独立標本とする．
  するとウィルコクソンのランク和統計量は
  \code{y[j]} が \code{x[i]} より大きくない全ての対
  \code{(x[i], y[j])} の数である．
  この統計量は \code{0} と \code{m * n} の間の値を取り，
  平均と分散はそれぞれ \code{m * n / 2} と
  \code{m * n * (m + n + 1) / 12} になる．

  もし最初の三つのどれかがベクトルならば，
  最長のベクトルになるまで三つの全ての組み合わせに対して計算を
  行うためにリサイクル規則が使われる．
}
\note{
  S-PLUS はウィルコクソン統計量の別の(しかし同値な)定義を使う：
  詳細は \code{\link{wilcox.test}} を見よ．
}
\section{警告}{
  これらの関数は一つの標本が大きい(数千以上)と
  大量のメモリーとスタックを使う
  (そしてもしスタックの限界を超えスタックのチェックが行われないと
  \R をクラッシュさえする)．
}
\source{
  これらは \code{cwilcox(k, m, n)} に基づき再帰的に計算される．
  これはサイズ \code{m} と \code{n} の標本から統計量 \code{k} 
  を用いた選択数であり，
  それ自体再帰的に計算されそして結果がキャッシュされる．
  それから \code{dwilcox} と \code{pwilcox} が
  \code{cwilcox} の適当な値を加える，そして \code{qwilcox}
  は逆関数に基づく．

  \code{rwilcox} はランクの無作為な置換を生成し統計量を評価する．
}
\author{Kurt Hornik}
\seealso{
  \code{\link{wilcox.test}} はデータから統計量を計算したり
  p 値を計算したりする．

  \emph{一変量}ウィルコクソン符号付きランク統計量の分布を
  含む標準分布については \link{Distributions}．
}
\examples{
require(graphics)

x <- -1:(4*6 + 1)
fx <- dwilcox(x, 4, 6)
Fx <- pwilcox(x, 4, 6)

layout(rbind(1,2), widths = 1, heights = c(3,2))
plot(x, fx, type = "h", col = "violet",
     main =  "Probabilities (density) of Wilcoxon-Statist.(n=6, m=4)")
plot(x, Fx, type = "s", col = "blue",
     main =  "Distribution of Wilcoxon-Statist.(n=6, m=4)")
abline(h = 0:1, col = "gray20", lty = 2)
layout(1) # 戻る

N <- 200
hist(U <- rwilcox(N, m = 4,n = 6), breaks = 0:25 - 1/2,
     border = "red", col = "pink", sub = paste("N =",N))
mtext("N * f(x),  f() = true \"density\"", side = 3, col = "blue")
 lines(x, N*fx, type = "h", col = "blue", lwd = 2)
points(x, N*fx, cex = 2)

## Quantile-Quantile Plot の方が良い
qqplot(U, qw <- qwilcox((1:N - 1/2)/N, m = 4, n = 6),
       main = paste("Q-Q-Plot of empirical and theoretical quantiles",
                     "Wilcoxon Statistic,  (m=4, n=6)", sep = "\n"))
n <- as.numeric(names(print(tU <- table(U))))
text(n+.2, n+.5, labels = tU, col = "red")
}
\keyword{distribution}

\encoding{UTF-8}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/stats/man/Wilcoxon.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{Wilcoxon}
%\alias{Wilcoxon}
%\alias{dwilcox}
%\alias{pwilcox}
%\alias{qwilcox}
%\alias{rwilcox}
%\title{Distribution of the Wilcoxon Rank Sum Statistic}
%\description{
%  Density, distribution function, quantile function and random
%  generation for the distribution of the Wilcoxon rank sum statistic
%  obtained from samples with size \code{m} and \code{n}, respectively.
%}
%\usage{
%dwilcox(x, m, n, log = FALSE)
%pwilcox(q, m, n, lower.tail = TRUE, log.p = FALSE)
%qwilcox(p, m, n, lower.tail = TRUE, log.p = FALSE)
%rwilcox(nn, m, n)
%}
%\arguments{
%  \item{x, q}{vector of quantiles.}
%  \item{p}{vector of probabilities.}
%  \item{nn}{number of observations. If \code{length(nn) > 1}, the length
%    is taken to be the number required.}
%  \item{m, n}{numbers of observations in the first and second sample,
%    respectively.  Can be vectors of positive integers.}
%  \item{log, log.p}{logical; if TRUE, probabilities p are given as log(p).}
%  \item{lower.tail}{logical; if TRUE (default), probabilities are
%    \eqn{P[X \le x]}, otherwise, \eqn{P[X > x]}.}
%}
%\value{
%  \code{dwilcox} gives the density,
%  \code{pwilcox} gives the distribution function,
%  \code{qwilcox} gives the quantile function, and
%  \code{rwilcox} generates random deviates.
%
%  The length of the result is determined by \code{nn} for
%  \code{rwilcox}, and is the maximum of the lengths of the
%  numerical arguments for the other functions.  
%  
%  The numerical arguments other than \code{nn} are recycled to the
%  length of the result.  Only the first elements of the logical
%  arguments are used.
%}
%\details{
%  This distribution is obtained as follows.  Let \code{x} and \code{y}
%  be two random, independent samples of size \code{m} and \code{n}.
%  Then the Wilcoxon rank sum statistic is the number of all pairs
%  \code{(x[i], y[j])} for which \code{y[j]} is not greater than
%  \code{x[i]}.  This statistic takes values between \code{0} and
%  \code{m * n}, and its mean and variance are \code{m * n / 2} and
%  \code{m * n * (m + n + 1) / 12}, respectively.
%
%  If any of the first three arguments are vectors, the recycling rule is
%  used to do the calculations for all combinations of the three up to
%  the length of the longest vector.
%}
%\note{
%  S-PLUS uses a different (but equivalent) definition of the Wilcoxon
%  statistic: see \code{\link{wilcox.test}} for details.
%}
%\section{Warning}{
%  These functions can use large amounts of memory and stack (and even crash
%  \R if the stack limit is exceeded and stack-checking is not in place)
%  if one sample is large (several thousands or more).
%}
%\source{
%  These are calculated via recursion, based on \code{cwilcox(k, m, n)},
%  the number of choices with statistic \code{k} from samples of size
%  \code{m} and \code{n}, which is itself calculated recursively and the
%  results cached.  Then \code{dwilcox} and \code{pwilcox} sum
%  appropriate values of \code{cwilcox}, and \code{qwilcox} is based on
%  inversion.
%
%  \code{rwilcox} generates a random permutation of ranks and evaluates
%  the statistic.
%}
%\author{Kurt Hornik}
%\seealso{
%  \code{\link{wilcox.test}} to calculate the statistic from data, find p
%  values and so on.
%
%  \link{Distributions} for standard distributions, including
%  \code{\link{dsignrank}} for the distribution of the
%  \emph{one-sample} Wilcoxon signed rank statistic.
%}
%\examples{
%require(graphics)
%
%x <- -1:(4*6 + 1)
%fx <- dwilcox(x, 4, 6)
%Fx <- pwilcox(x, 4, 6)
%
%layout(rbind(1,2), widths = 1, heights = c(3,2))
%plot(x, fx, type = "h", col = "violet",
%     main =  "Probabilities (density) of Wilcoxon-Statist.(n=6, m=4)")
%plot(x, Fx, type = "s", col = "blue",
%     main =  "Distribution of Wilcoxon-Statist.(n=6, m=4)")
%abline(h = 0:1, col = "gray20", lty = 2)
%layout(1) # set back
%
%N <- 200
%hist(U <- rwilcox(N, m = 4,n = 6), breaks = 0:25 - 1/2,
%     border = "red", col = "pink", sub = paste("N =",N))
%mtext("N * f(x),  f() = true \"density\"", side = 3, col = "blue")
% lines(x, N*fx, type = "h", col = "blue", lwd = 2)
%points(x, N*fx, cex = 2)
%
%## Better is a Quantile-Quantile Plot
%qqplot(U, qw <- qwilcox((1:N - 1/2)/N, m = 4, n = 6),
%       main = paste("Q-Q-Plot of empirical and theoretical quantiles",
%                     "Wilcoxon Statistic,  (m=4, n=6)", sep = "\n"))
%n <- as.numeric(names(print(tU <- table(U))))
%text(n+.2, n+.5, labels = tU, col = "red")
%}
%\keyword{distribution}
%
%\encoding{UTF-8}
%